package ProjectTankHard.Generics;

import ProjectTankHard.DrawningObjects.DrawningTankBase;
import ProjectTankHard.MovementStrategy.IMoveableObject;

import java.awt.*;
import java.awt.image.BufferedImage;

public class TanksGenericCollection<T extends DrawningTankBase, U extends IMoveableObject> {
    private final int _pictureWidth;
    private final int _pictureHeight;
    private final int _placeSizeWidth = 150;
    private final int _placeSizeHeight = 100;
    private final SetGeneric<T> _collection;
    public TanksGenericCollection(int picWidth, int picHeight){
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = new SetGeneric<T>(width * height);
    }

    public int Insert(T obj){
        if (obj == null) return -1;

        return _collection.Insert(obj);
    }

    public boolean Remove(int position){
        return _collection.Remove(position);
    }
    public U GetU(int pos){
        T ans = _collection.Get(pos);
        if(ans == null)
            return null;
        return (U)ans.GetMoveableObject();
    }

    private void DrawBackground(Graphics g)
    {
        g.setColor(Color.BLACK);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {
                g.drawLine(i * _placeSizeWidth, j * _placeSizeHeight,
                        i * _placeSizeWidth + _placeSizeWidth / 2,
                        j * _placeSizeHeight);
            }
            g.drawLine(i * _placeSizeWidth, 0,
                    i * _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }

    private void DrawObjects(Graphics g)
    {
        for (int i = 0; i < _collection.Count; i++)
        {
            DrawningTankBase tank = _collection.Get(i);
            if (tank != null)
            {
                int columnsCount = _pictureHeight / _placeSizeHeight;

                int colIndex = i % columnsCount;
                int rowIndex = i / columnsCount;

                tank.SetPosition(colIndex * _placeSizeWidth, rowIndex * _placeSizeHeight);
                tank.DrawTank((Graphics2D) g);
            }
        }
    }
    public BufferedImage ShowTanks()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics gr = bmp.createGraphics();
        DrawBackground(gr);
        DrawObjects(gr);
        return bmp;
    }
}